/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"fmt"
	"strconv"

	"github.com/Nerzal/gocloak/v7"
	"github.com/spf13/cobra"
)

func NewCreateRealmCommand(rCli *RealmClient) *cobra.Command {
	// createCmd represents the create command
	ro := RealmOptions{}

	createCmd := &cobra.Command{
		Use:   "realm",
		Short: "A brief description of your command",
		Long: `A longer description that spans multiple lines and likely contains examples
and usage of using your command. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.Background()
			token, err := rCli.Credentials.GetToken(ctx, rCli.URL, rCli.Name)
			if err != nil {
				return err
			}

			realm := ro.GetRealm()
			res, err := rCli.GetClient().CreateRealm(ctx, token.AccessToken, realm)
			if err != nil {
				return err
			}

			fmt.Println(fmt.Sprintf("Creating Realm: %s", res))

			return nil
		},
	}

	defaultLocales := []string{
		"fr",
		"en",
	}

	createCmd.Flags().BoolVarP(&ro.Enabled, "enabled", "", true, "Enable Realm")

	createCmd.Flags().BoolVarP(&ro.RegistrationEmailAsUsername, "email-as-username", "", false, "")
	createCmd.Flags().BoolVarP(&ro.InternationalizationEnabled, "internationalization", "", true, "")
	createCmd.Flags().StringArrayVarP(&ro.SupportedLocales, "supported-locales", "", defaultLocales, "")
	createCmd.Flags().StringVarP(&ro.DefaultLocale, "default-locale", "", "fr", "")

	createCmd.Flags().BoolVarP(&ro.RegistrationAllowed, "registration", "", true, "")
	createCmd.Flags().BoolVarP(&ro.LoginWithEmailAllowed, "login-email", "", true, "")
	createCmd.Flags().BoolVarP(&ro.VerifyEmail, "verify-email", "", true, "")
	createCmd.Flags().BoolVarP(&ro.RememberMe, "rememberMe", "", true, "")
	createCmd.Flags().BoolVarP(&ro.ResetPasswordAllowed, "reset-password", "", true, "")
	createCmd.Flags().BoolVarP(&ro.EditUsernameAllowed, "edit-username", "", false, "")
	createCmd.Flags().BoolVarP(&ro.DuplicateEmailsAllowed, "duplicate-emails", "", false, "")

	createCmd.Flags().StringVarP(&ro.Name, "name", "", "", "")
	createCmd.Flags().StringVarP(&ro.AdminTheme, "admin-theme", "", "keycloak", "")
	createCmd.Flags().StringVarP(&ro.EmailTheme, "email-theme", "", "keycloak", "")
	createCmd.Flags().StringVarP(&ro.LoginTheme, "login-theme", "", "keycloak", "")
	createCmd.Flags().StringVarP(&ro.AccountTheme, "account-theme", "", "keycloak", "")

	createCmd.Flags().BoolVarP(&ro.SMTPServer.Auth, "smtp-auth", "", true, "")
	createCmd.Flags().StringVarP(&ro.SMTPServer.User, "smtp-user", "", "", "")
	createCmd.Flags().StringVarP(&ro.SMTPServer.From, "smtp-from", "", "", "")
	createCmd.Flags().StringVarP(&ro.SMTPServer.Password, "smtp-password", "", "", "")
	createCmd.Flags().StringVarP(&ro.SMTPServer.Host, "smtp-host", "", "mail.indie.host", "")
	createCmd.Flags().BoolVarP(&ro.SMTPServer.SSL, "smtp-ssl", "", false, "")
	createCmd.Flags().Int32VarP(&ro.SMTPServer.Port, "smtp-port", "", 587, "")
	createCmd.Flags().BoolVarP(&ro.SMTPServer.Starttls, "smtp-starttls", "", true, "")

	return createCmd
}

func (ro *RealmOptions) GetRealm() gocloak.RealmRepresentation {
	if len(ro.SMTPServer.From) == 0 {
		ro.SMTPServer.From = ro.SMTPServer.User
	}

	smtp := map[string]string{
		"auth":     strconv.FormatBool(ro.SMTPServer.Auth),
		"user":     ro.SMTPServer.User,
		"from":     ro.SMTPServer.From,
		"password": ro.SMTPServer.Password,
		"host":     ro.SMTPServer.Host,
		"Host":     ro.SMTPServer.Host,
		"port":     strconv.Itoa(int(ro.SMTPServer.Port)),
		"Port":     strconv.Itoa(int(ro.SMTPServer.Port)),
		"starttls": strconv.FormatBool(ro.SMTPServer.Starttls),
	}

	return gocloak.RealmRepresentation{
		Realm:                       &ro.Name,
		SMTPServer:                  &smtp,
		Enabled:                     &ro.Enabled,
		RegistrationEmailAsUsername: &ro.RegistrationEmailAsUsername,
		RegistrationAllowed:         &ro.RegistrationAllowed,
		LoginWithEmailAllowed:       &ro.LoginWithEmailAllowed,
		VerifyEmail:                 &ro.VerifyEmail,
		RememberMe:                  &ro.RememberMe,
		ResetPasswordAllowed:        &ro.ResetPasswordAllowed,
		EditUsernameAllowed:         &ro.EditUsernameAllowed,
		DuplicateEmailsAllowed:      &ro.DuplicateEmailsAllowed,
		EmailTheme:                  &ro.EmailTheme,
		AdminTheme:                  &ro.AdminTheme,
		LoginTheme:                  &ro.LoginTheme,
		AccountTheme:                &ro.AccountTheme,
		DefaultLocale:               &ro.DefaultLocale,
		DisplayName:                 &ro.DisplayName,
		DisplayNameHTML:             &ro.DisplayNameHTML,
		InternationalizationEnabled: &ro.InternationalizationEnabled,
		SupportedLocales:            &ro.SupportedLocales,
	}

}

type RealmOptions struct {
	Name string
	*RealmClient
	Enabled                     bool
	RegistrationEmailAsUsername bool
	RegistrationAllowed         bool
	LoginWithEmailAllowed       bool
	VerifyEmail                 bool
	RememberMe                  bool
	ResetPasswordAllowed        bool
	EditUsernameAllowed         bool
	DuplicateEmailsAllowed      bool
	EmailTheme                  string
	AdminTheme                  string
	LoginTheme                  string
	AccountTheme                string
	SMTPServer                  SMTPOptions
	DefaultLocale               string
	SupportedLocales            []string
	DisplayName                 string
	DisplayNameHTML             string
	InternationalizationEnabled bool
}

type SMTPOptions struct {
	Auth     bool
	User     string
	Password string
	Host     string
	From     string
	SSL      bool
	Port     int32
	Starttls bool
}

func (r *RealmOptions) Validate() error {
	return nil
}

func (r *RealmOptions) Complete() error {
	return nil
}

func (r *RealmOptions) Run() error {
	return nil
}
