/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"fmt"
	"os"
	"strings"

	"github.com/spf13/cobra"
	"github.com/spf13/pflag"

	"github.com/Nerzal/gocloak/v7"
	homedir "github.com/mitchellh/go-homedir"
	"github.com/spf13/viper"
)

var cfgFile string

const (
	// The name of our config file, without the file extension because viper supports many different config file languages.
	defaultConfigFilename = ".keycloak-cli"

	// The environment variable prefix of all environment variables bound to our command line flags.
	// For example, --number is bound to STING_NUMBER.
	envPrefix = "KEYCLOAK"
)

var home = os.Getenv("HOME")

type RealmClient struct {
	AuthRealm   string
	URL         string
	Token       *gocloak.JWT
	Credentials *Credentials
	Name        string
}

func (r *RealmClient) GetClient() gocloak.GoCloak {
	return gocloak.NewClient(r.URL)
}

// rootCmd represents the base command when called without any subcommands
func NewRootCommand() *cobra.Command {
	//	ro := NewRealmOptions()
	rCli := &RealmClient{}
	rCli.Credentials = &Credentials{}

	rootCmd := &cobra.Command{
		Use:   "keycloak-cli",
		Short: "A brief description of your application",
		Long:  "",
		PersistentPreRunE: func(cmd *cobra.Command, args []string) error {
			// You can bind cobra and viper in a few locations, but PersistencePreRunE on the root command works well
			return initConfig(cmd)
		},
		Run: func(cmd *cobra.Command, args []string) {
		},
	}

	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is $HOME/.keycloak-cli.yaml)")
	// Cobra also supports local flags, which will only run
	// when this action is called directly.
	rootCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")
	rootCmd.PersistentFlags().StringVarP(&rCli.URL, "url", "", "", "Base URL for Keycloak, facultative if you have a "+home+"/.keycloak-cli.yaml")
	rootCmd.PersistentFlags().StringVarP(&rCli.AuthRealm, "auth-realm", "r", "", "")
	rootCmd.PersistentFlags().StringVarP(&rCli.Credentials.Username, "auth-username", "u", "", "")
	rootCmd.PersistentFlags().StringVarP(&rCli.Credentials.Password, "auth-password", "p", "", "")
	rootCmd.PersistentFlags().StringVarP(&rCli.Name, "realm", "", "", "")
	//	rootCmd.PersistentFlags().StringVarP(&configFile, "configFile", "c", home+"", "configuration file, default is "+home+"$HOME/.keycloak-cli.yaml")

	//	if len(rCli.Name) == 0 {
	//		path, err := os.Getwd()
	//		if err != nil {
	//			log.Println(err)
	//		}

	//fmt.Println(path) // for example /home/user

	// TODO check if in /data/domains/$domain
	//	path = `data/domains/liiib-re`

	//	realmName := filepath.Base(path)

	//	rCli.Name = realmName
	//	fmt.Println(rCli.Name)

	//}

	rootCmd.AddCommand(NewCreateCommand(rCli))
	rootCmd.AddCommand(NewEmailCommand())

	return rootCmd
}

// Execute adds all child commands to the root command and sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	rootCmd := NewRootCommand()

	if err := rootCmd.Execute(); err != nil {
		fmt.Fprintln(os.Stderr, err)
		os.Exit(-1)
	}
}

// initConfig reads in config file and ENV variables if set.
func initConfig(cmd *cobra.Command) error {
	v := viper.New()

	if cfgFile != "" {
		// Use config file from the flag.
		v.SetConfigFile(cfgFile)
	} else {
		// Find home directory.
		home, err := homedir.Dir()
		if err != nil {
			return err
		}

		// Search config in home directory with name ".keycloak-cli" (without extension).
		v.AddConfigPath(home)
		v.SetConfigName(".keycloak-cli")
	}

	//	v.AutomaticEnv() // read in environment variables that match

	// If a config file is found, read it in.
	err := v.ReadInConfig()
	if err == nil {
		fmt.Println("Using config file:", v.ConfigFileUsed())
	}
	// When we bind flags to environment variables expect that the
	// environment variables are prefixed, e.g. a flag like --number
	// binds to an environment variable STING_NUMBER. This helps
	// avoid conflicts.
	// v.SetEnvPrefix(envPrefix)

	// Bind to environment variables
	// Works great for simple config names, but needs help for names
	// like --favorite-color which we fix in the bindFlags function
	v.AutomaticEnv()

	// Bind the current command's flags to viper
	bindFlags(cmd, v)

	return nil
}

// Bind each cobra flag to its associated viper configuration (config file and environment variable)
func bindFlags(cmd *cobra.Command, v *viper.Viper) {
	cmd.Flags().VisitAll(func(f *pflag.Flag) {
		// Environment variables can't have dashes in them, so bind them to their equivalent
		// keys with underscores, e.g. --favorite-color to STING_FAVORITE_COLOR
		if strings.Contains(f.Name, "-") {
			envVarSuffix := strings.ToUpper(strings.ReplaceAll(f.Name, "-", "_"))
			// v.BindEnv(f.Name, fmt.Sprintf("%s_%s", envPrefix, envVarSuffix))
			v.BindEnv(envVarSuffix)
		}
		// Apply the viper config value to the flag when the flag is not set and viper has a value
		if !f.Changed && v.IsSet(f.Name) {
			val := v.Get(f.Name)
			cmd.Flags().Set(f.Name, fmt.Sprintf("%v", val))
		}
	})
}

type Credentials struct {
	Username string
	Password string
}

func (c *Credentials) GetToken(ctx context.Context, url, realm string) (*gocloak.JWT, error) {
	client := gocloak.NewClient(url)
	token, err := client.LoginAdmin(ctx, c.Username, c.Password, realm)
	if err != nil {
		return token, err
	}
	return token, nil
}

type OIDCOptions struct {
	// Access:
	//   configure: "",
	//   manage: "",
	//   view: "",
	BaseURL  string
	ClientID string
	// ConsentRequired: "",
	// Description:     "",
	Enabled bool
	// Origin:          "",
	Name string
	// Secret:          "",
	RootURL                 string
	ClientAuthenticatorType string
	RedirectURIs            []string
	WebOrigins              []string
}

type SAMLClientOptions struct {
}
